@@ -3,13 +3,14 @@ package Pod::Xhtml;
 use strict;
 use Pod::Parser;
 use Pod::ParseUtils;
+use Carp;
 use vars qw/@ISA %COMMANDS %SEQ $VERSION $FirstAnchorId $ContentSuffix/;
 
 $FirstAnchorId = "TOP";
 $ContentSuffix = "-CONTENT";
 
 @ISA = qw(Pod::Parser);
-($VERSION) = ('$Revision: 1.52 $' =~ m/([\d\.]+)/);
+($VERSION) = ('$Revision: 1.56 $' =~ m/([\d\.]+)/);
 
 # recognized commands
 %COMMANDS = map { $_ => 1 } qw(pod head1 head2 head3 head4 item over back for begin end);
@@ -59,6 +60,9 @@ sub initialize {
 	$self->{FragmentOnly} = 0 unless defined $self->{FragmentOnly};
 	$self->{HeadText} = $self->{BodyOpenText} = $self->{BodyCloseText} = '';
 	$self->{LinkParser} ||= new Pod::Hyperlink;
+	$self->{TopHeading} ||= 1;
+	$self->{TopHeading} = int $self->{TopHeading}; # heading level must be an integer
+	croak "TopHeading must be greater than zero" if $self->{TopHeading} < 1; # (prevent negative heading levels)
 	$self->SUPER::initialize();
 }
 
@@ -197,13 +201,16 @@ sub _addCommand {
 	for ($command) {
 		my $data_para = @{$self->{dataSections}}; # inside a data paragraph?
 		/^head1/ && !$data_para && do {
+			my $top_heading = 'h'. $self->{TopHeading};
+			$top_heading = 'h1' if !$self->{FragmentOnly}; # ignore TopHeading when not in fragment mode
+
 			# if ANY sections are open then close the previously opened div
 			$self->{buffer} .= "\n</div>\n" unless ( @{ $self->{sections} } == 1 );
 			
 			$anchor = $self->_addSection( 'head1', $paragraph );
 			my $anchorContent = $self->_addSection( '', $paragraph . $ContentSuffix);
-
-			$self->{buffer} .= qq(<h1 id="$anchor">$paragraph</h1>)
+			
+			$self->{buffer} .= qq(<$top_heading id="$anchor">$paragraph</$top_heading>)
 					.($self->{TopLinks} ? $self->{TopLinks} : '')."\n"
 					."<div id=\"$anchorContent\">\n";
 
@@ -212,7 +219,10 @@ sub _addCommand {
 		};
 		/^head([234])/ && !$data_para && do {
 			my $head_level = $1;
-
+			if($self->{FragmentOnly}){
+				$head_level += ($self->{TopHeading} - 1);
+				$head_level = 6 if $head_level > 6;
+			}
 			# if ANY sections are open then close the previously opened div
 			$self->{buffer} .= "\n</div>\n" unless ( @{ $self->{sections} } == 1 );
 
@@ -228,7 +238,7 @@ sub _addCommand {
 				last;
 			}
 
-			$self->{listHasItems}[-1] = 1;
+			$self->{listHasItems}[-1]++;
 			$self->{listCurrentParas}[-1] = 0;
 
 			# is this the first item in the list?
@@ -291,10 +301,11 @@ sub _addCommand {
 			push @{$self->{listCurrentParas}}, 0;
 		};
 		/^back/ && !$data_para && do {
+			my $listItems = pop @{$self->{listHasItems}};
 			if (--$self->{inList} < 0) {
 				warn "=back commands don't balance =overs at $self->{_INFILE} line $line\n";
 				last;
-			} elsif ($self->{listHasItems} == 0) {
+			} elsif ($listItems == 0) {
 				warn "empty list at $self->{_INFILE} line $line\n";
 				last;
 			} elsif (@{$self->{listKind}} && $self->{listKind}[-1] == 1) {
@@ -344,7 +355,6 @@ sub _addCommand {
 				}
 			}
 
-			pop @{$self->{listHasItems}};
 			pop @{$self->{listKind}};
 			pop @{$self->{listCurrentParas}};
 			last;
@@ -634,6 +644,17 @@ sub seqL {
 		my $targ = $self->{LinkParser}->node;
 		my $text = $self->{LinkParser}->text;
 		$string = qq(<a href="$targ">$text</a>);
+	} elsif ($kind =~ m/^bounceurl:(.+)$/) {
+		# Our link-parser has decided that the link should be handled by a particular URL
+		my $url = $1;
+		$url = _htmlEscape( $url ); # since the URL may contain ampersands
+		$string = $self->{LinkParser}->markup;
+		if ($string =~ m/P<.+>/) {	# when there's no alternative text we get P, and maybe Q
+			$string =~ s|Q<(.+?)>|<strong class="pod_xhtml_bounce_url_text">$1</strong>|;
+			$string =~ s|P<(.+?)>|<a class="pod_xhtml_bounce_url" href="$url">$1</a>|;
+		} else {
+			$string =~ s|Q<(.+?)>|<a class="pod_xhtml_bounce_url" href="$url">$1</a>|;
+		}
 	} elsif ($page eq '') {	# a link to this page
 		# Post-process these links so we can things up to the correct sections
 		my $targ = $self->{LinkParser}->node;
@@ -797,6 +818,21 @@ Default: 0. If 1, we only produce an XHTML fragment (suitable for use as a
 server-side include etc). There is no HEAD element nor any BODY or HTML
 tags. Any text added with the add*Text methods will B<not> be output.
 
+
+=item TopHeading
+
+Allows you to set the starting heading level when in fragment mode.
+For example, if your document already has h1 tags and you want the
+generated POD to nest inside the outline, you can specify
+
+	TopHeading => 2
+
+and C<=head1> will be tagged with h2 tags, C<=head3> with h3, and so
+on.
+
+Note that XHTML doesn't allow for heading tags past h6, so h7 and up
+will be translated to h6 as necessary.
+
 =item TopLinks
 
 At each section head this text is added to provide a link back to the top.
@@ -810,6 +846,10 @@ An object that parses links in the POD document. By default, this is a regular
 Pod::Hyperlink object. Any user-supplied link parser must conform the the
 Pod::Hyperlink API.
 
+This module works with a L<Pod::Hyperlink::BounceURL> link parser and generates
+hyperlinks as 'a' elements with a class of 'pod_xhtml_bounce_url'. The optional
+text giving the "node" is enclosed in a 'strong' element with a class of 'pod_xhtml_bounce_url_text'
+
 =back
 
 =head1 RATIONALE
